﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using ColossalFramework;
using ColossalFramework.Globalization;
using ColossalFramework.IO;
using ColossalFramework.Packaging;
using ICities;
using UEObj = UnityEngine.Object;

namespace FixColorCorrectionsMonitor
{
	public class FixColorCorrectionsMonitor : LoadingExtensionBase
	{
		private FileSystemWatcher _fileSystemWatcher;
		private MethodInfo _callPrivateRefresh;

		public override void OnCreated(ILoading loading)
		{
			Log.Message("initializing...");

			_callPrivateRefresh = typeof (ColorCorrectionManager).GetMethod("Refresh",
				BindingFlags.Instance | BindingFlags.NonPublic);

			if (_callPrivateRefresh == null)
			{
				Log.Message("couldn't find ColorCorrectionManager.Refresh()");
				return;
			}

			_fileSystemWatcher = new FileSystemWatcher(ColorCorrectionManager.colorCorrectionsPath, "*.png");
			_fileSystemWatcher.Created += FileSystemWatcherOnCreated;
			_fileSystemWatcher.Changed += FileSystemWatcherOnChanged;
			_fileSystemWatcher.EnableRaisingEvents = true;
			Log.Message("watching: " + ColorCorrectionManager.colorCorrectionsPath);
		}
		
		private void FileSystemWatcherOnCreated(object sender, FileSystemEventArgs fileSystemEventArgs)
		{
			Refresh("OnCreated");
		}

		private void FileSystemWatcherOnChanged(object sender, FileSystemEventArgs fileSystemEventArgs)
		{
			Refresh("OnChanged");
		}

		private void Refresh(string source)
		{
			try
			{
				Log.Message(source + ", invoking ColorCorrectionManager.Refresh()");
				var instance = Singleton<ColorCorrectionManager>.instance;
				object sender = null;
				ReporterEventArgs args = null;
				_callPrivateRefresh.Invoke(instance, new[] { sender, args });
			}
			catch (Exception ex)
			{
				Log.Message("error: " + ex.ToString());
			}
		}

		public override void OnReleased()
		{
			if (_fileSystemWatcher != null)
			{
				_fileSystemWatcher.Dispose();
				_fileSystemWatcher = null;
			}
		}
	}
}